/*
 * FILE:    mbus_control.c
 * PROGRAM: SIP_MBUS - controller
 * AUTHOR:  KyungTae Kim 
 * VERSION: 1.0
 *
 * Copyright (c) 1999-2001 Columbia University 
 * All rights reserved.
 */
 
#include "config_unix.h"
#include "config_win32.h"
#include "debug.h"
#include "mbus.h"
#include "memory.h"
#include "mbus_parser.h"
#include "mbus_control.h"

#define TRUE 	1
#define FALSE 	0

extern int 	 should_exit;
extern char	*u_addr, *e_addr;
static struct mbus *mbus = NULL;
struct timeval timeout;

/* Mbus command reception function type */
typedef void (*mbus_rx_proc)(char *srce, char *args, struct mbus *data);

/* Tuple to associate string received with it's parsing fn */
typedef struct {
  const char   *rxname;
  mbus_rx_proc  rxproc;
} mbus_cmd_tuple;

static char *wait_token;
static char *wait_addr;

static char *addr_rat_control;
static char *addr_rat_ui;
static char *addr_rat_engine;

int control_firsttime = TRUE;;
int engine_firsttime = TRUE;;
int ui_firsttime = TRUE;

/* to get a respond message */
void wait_result() {
  timeout.tv_sec  = 0;
  timeout.tv_usec = 300000;
  mbus_recv(mbus, mbus, &timeout);
}

/* 
 * sending rtp.query mbus command to rat_engine 
 * if you send it to control or ui, it should be killed
 * mbus_engine of rat will reply rtp information such as
 * application name, e-mail address, phone number etc.
 */
void send_rtp_query() {
  mbus_qmsgf(mbus, addr_rat_engine, TRUE, "rtp.query", "");
  mbus_send(mbus);
}

/* 
 * sending audio.query mbus command to rat_engine 
 * if you send it to control or ui, it should be killed also
 * mbus_engine of rat will reply audio information such as
 * audio codec, bps, packetization etc.
 */
void send_audio_query() {
  mbus_qmsgf(mbus, addr_rat_engine, TRUE, "audio.query", "");
  mbus_send(mbus);
}

/* 
 * for synchronization
 * in future use
 */
void mbus_control_wait_init(char *token)
{
  wait_token = token;
  wait_addr  = NULL;
}

/* 
 * for synchronization
 * in future use
 */
char *mbus_control_wait_done(void)
{
  return wait_addr;
}

/* 
 * When RAT exit abnornally, the main() will cleanup 
 * for debugging, it is used
 * in release verision, should_exit = FALSE or erase it
 */
static void rx_mbus_quit(char *srce, char *args, struct mbus *data)
{
  UNUSED(srce);
  UNUSED(args);
  UNUSED(data);

#ifdef DEBUG
  fprintf(stderr, "Got quit %s\n", srce);
#endif

  /* We mark ourselves as ready to exit. The main() will */
  /* cleanup and terminate all our subprocesses.         */
  should_exit = TRUE;
}

/* 
 * If RAT engines exit, it clear data structure for them
 */
static void rx_mbus_bye(char *srce, char *args, struct mbus *data)
{
  if(strcmp(srce, addr_rat_ui) == 0) {
	ui_firsttime = TRUE;
	addr_rat_ui = NULL;
  } else if (strcmp(srce, addr_rat_engine) == 0) {
	engine_firsttime = TRUE;
	addr_rat_engine = NULL;
  } else if (strcmp(srce, addr_rat_control) == 0) {
	control_firsttime = TRUE;
	addr_rat_control = NULL;
  }
  UNUSED(args);
  UNUSED(data);
}

/* 
 * for synchronization
 * in future use
 */
static void rx_mbus_waiting(char *srce, char *args, struct mbus *data)
{
	UNUSED(srce);
	UNUSED(args);
	UNUSED(data);
}

/*
 * For rtp.addr command, the engine of RAT respond with its address
 * in current version, it is not used
 */
static void rx_rtp_addr(char *srce, char *args, struct mbus *data)
{
	struct mbus_parser	*mp;
	char			*t;

	UNUSED(data);
	UNUSED(srce);

	mp = mbus_parse_init(args);
	mbus_parse_str(mp, &t);
#ifdef DEBUG
	fprintf(stderr, "rtp_addr: %s\n", mbus_decode_str(t));
#endif
	mbus_parse_done(mp);
#ifdef DEBUG
	fprintf(stderr, "in rx_rtp_addr, addr_rat_engine: %s\n", addr_rat_engine);
#endif
}

/*
 * sipc changes the remote ip-address, src/dst port number
 */ 
void rtp_addr_change(char *args[]) {
  char *addr;
  int src, dst, ttl;

  // how about receiving "ip src dst ttl" as one string
  addr = strdup(args[0]);
  src = atoi(args[1]);
  dst = atoi(args[2]);
  ttl = atoi(args[3]);

  addr = mbus_encode_str(addr);

  /* change the address of rat ui */

  if(addr_rat_ui != NULL) {
    mbus_qmsgf(mbus, addr_rat_ui, TRUE, "rtp.addr.change", "%s %d %d %d", addr, src, dst, ttl);
	mbus_send(mbus);
#ifdef DEBUG
    fprintf(stderr, "Sending rat.addr.change to %s\n", addr_rat_ui);
#endif
  } else {
#ifdef DEBUG
    fprintf(stderr, "NO rat-ui\n");
#endif
  }

  /* change the address of rat engine */
  if(addr_rat_engine != NULL) {
    mbus_qmsgf(mbus, addr_rat_engine, TRUE, "rtp.addr.change", "%s %d %d %d", addr, src, dst, ttl);
	mbus_send(mbus);
    send_rtp_query();
	send_audio_query();
#ifdef DEBUG
    fprintf("Sending rat.addr.change to %s\n", addr_rat_engine);
#endif
  } else {
#ifdef DEBUG
    fprintf(stderr, "NO rat-engine\n");
#endif
  }
}

/* toggle mute/talk of RAT
 * mute = 1: mute
 * mute = 0: talk
 */
void rat_talk(char *args) {
  int mute;
  
  mute = atoi(args);
  /* turn on/off the volume of rat-engine  */
  if(addr_rat_engine != NULL) {
	mbus_qmsgf(mbus, addr_rat_engine, TRUE, "audio.input.mute", "%d", mute);
    mbus_send(mbus);
#ifdef DEBUG
    fprintf("Sending audio.input.mute to %s\n", addr_rat_engine);
#endif
	wait_result();
  }

  /* turn on/off the volume of rat-ui  */
  if(addr_rat_ui != NULL) {
	mbus_qmsgf(mbus, addr_rat_ui, TRUE, "audio.input.mute", "%d", mute);
    mbus_send(mbus);
#ifdef DEBUG
    fprintf(stderr, "Sending audio.input.mute to %s\n", addr_rat_ui);
#endif
	wait_result();
  }
}

/* 
 * change audio codec of RAT
 */
void tool_rat_codec(char *args)
{
  char *name;
  char *freq = "mono";
  char *chan = "8k";

  name = strdup(args);

  if ((strcmp(name, "l16") != 0) && \
	  (strcmp(name, "pcmu") != 0) && \
	  (strcmp(name, "pcma") != 0) && \
	  (strcmp(name, "dvi") != 0) && \
	  (strcmp(name, "gsm") != 0) && \
	  (strcmp(name, "lpc") != 0) )
  {
#ifdef DEBUG
	fprintf(stderr, "wrong codec name: %s\n", name);
#endif
	name = "lpc";
  }

  name = mbus_encode_str(name);
  freq = mbus_encode_str(freq);
  chan = mbus_encode_str(chan);

  if(addr_rat_engine != NULL) {
	mbus_qmsgf(mbus, addr_rat_engine, TRUE, "tool.rat.codec", "%s %s %s", name, freq, chan);
	mbus_send(mbus);
#ifdef DEBUG
    fprintf(stderr, "Sending tool.rat.codec to %s\n", addr_rat_engine);
#endif
  }
}

void tool_codec_rate(char *args) {
  int packetization;
  
  packetization = atoi(args);
  /* turn on/off the volume of rat-engine  */
  if(addr_rat_engine != NULL) {
	mbus_qmsgf(mbus, addr_rat_engine, TRUE, "tool.rat.rate", "%d", packetization);
    mbus_send(mbus);
#ifdef DEBUG
    fprintf(stderr, "Sending tool.rat.rate to %s\n", addr_rat_engine);
#endif
  }
  if(addr_rat_ui != NULL) {
	mbus_qmsgf(mbus, addr_rat_ui, TRUE, "tool.rat.rate", "%d", packetization);
    mbus_send(mbus);
#ifdef DEBUG
    fprintf(stderr, "Sending tool.rat.rate to %s\n", addr_rat_ui);
#endif
  }
}

/* 
 * rtp.source.tool mbus command receiving function
 */
static void rx_rtp_source_tool(char *srce, char *args, struct mbus *data)
{
	struct mbus_parser	*mp;
	char			*t;

	UNUSED(data);
	UNUSED(srce);

	mp = mbus_parse_init(args);
	mbus_parse_str(mp, &t);
#ifdef DEBUG
	fprintf(stderr, "rtp_source_tool: %s\n", mbus_decode_str(t));
#endif
	mbus_parse_done(mp);
}


/* 
 * tool.rat.codec mbus command receiving function
 */
static void rx_tool_rat_codec(char *srce, char *args, struct mbus *data)
{
	struct mbus_parser	*mp;
	char			*t;

	UNUSED(data);
	UNUSED(srce);

	mp = mbus_parse_init(args);
	mbus_parse_str(mp, &t);
#ifdef DEBUG
	fprintf(stderr, "tool_rat_codec: %s\n\n", mbus_decode_str(t));
#endif
	mbus_parse_done(mp);
}

/* 
 * for synchronization
 * in future use
 */
static void rx_mbus_go(char *srce, char *args, struct mbus *data)
{
	struct mbus_parser	*mp;
	char			*t;

	UNUSED(data);

	mp = mbus_parse_init(args);
	mbus_parse_str(mp, &t);
	if (strcmp(mbus_decode_str(t), wait_token) == 0) {
		wait_addr = xstrdup(srce);
	}
	mbus_parse_done(mp);
}

/* 
 * mbus.hello command process function
 * When it is received, the data stucture for them is initialized
 */
static void rx_mbus_hello(char *srce, char *args, struct mbus *data)
{
	UNUSED(args);
	if (strstr(srce, "media:audio module:control app:rat") != NULL) {
	  if(control_firsttime) {
		control_firsttime = FALSE;
	  }
	  addr_rat_control = strdup(srce);
	} else if(strstr(srce, "media:audio module:engine app:rat") != NULL) {
	  if(engine_firsttime) {
		engine_firsttime = FALSE;
	  }
	  addr_rat_engine = strdup(srce);
	} else if(strstr(srce, "media:audio module:ui app:rat") != NULL) {
	  if(ui_firsttime) {
		ui_firsttime = FALSE;
	  }
	  addr_rat_ui = strdup(srce);
	}
}

/*
 * definition of mbus command process functions
 */
const mbus_cmd_tuple control_cmds[] = {
  { "mbus.quit",                             rx_mbus_quit },
  { "mbus.bye",                              rx_mbus_bye },
  { "mbus.waiting",                          rx_mbus_waiting },
  { "mbus.go",                               rx_mbus_go },
  { "mbus.hello",                            rx_mbus_hello },
  { "rtp.addr",                              rx_rtp_addr },
  { "rtp.source.tool",                       rx_rtp_source_tool },
  { "tool.rat.codec",                        rx_tool_rat_codec },
};

#define NUM_CONTROL_CMDS sizeof(control_cmds)/sizeof(control_cmds[0])

/* 
 * a function for receiving all mbus command
 */
void mbus_control_rx(char *srce, char *cmnd, char *args, void *data)
{
  uint32_t i;

  for (i=0; i < NUM_CONTROL_CMDS; i++) {
	if (strcmp(control_cmds[i].rxname, cmnd) == 0) {
      control_cmds[i].rxproc(srce, args, (struct mbus *)data);
	  mbus = (struct mbus*)data;
	  return;
	} 
  }
#ifdef DEBUG
  fprintf(stderr, "Unknown mbus command: %s (%s)\n", cmnd, args);
#endif
}
