/*
 * FILE: sip_mbus.c
 * PROGRAM: sip-mbus interface
 * AUTHOR: Xiaotao, KyungTae Kim
 *
 * This is the sip-mbus interface
 *
 * Copyright (c) 2001 Columbia University
 * All right reserved.
 */

#include <tcl.h>
#include <stdlib.h>
#include <assert.h>
#include "sip_mbus_control.h"

EXTERN int		Sipcmbus_Init	_ANSI_ARGS_((Tcl_Interp *interp));

int sipc_mbus_control(ClientData dummy, Tcl_Interp *interp, int argc, char **argv);
int sipc_mbus_rtp_addr(ClientData dummy, Tcl_Interp *interp, int argc, char **argv);
int sipc_mbus_talk(ClientData dummy, Tcl_Interp *interp, int argc, char **argv);
int sipc_mbus_codec(ClientData dummy, Tcl_Interp *interp, int argc, char **argv);
int sipc_mbus_packetization(ClientData dummy, Tcl_Interp *interp, int argc, char **argv);

typedef struct {
    char *name;                 /* Name of command. */
    Tcl_CmdProc *cmdProc;       /* Command procedure. */
} MbusCmd;

static MbusCmd commands[] = {
  {"mbus_init",            sipc_mbus_control},
  {"mbus_rtp_addr",        sipc_mbus_rtp_addr},
  {"mbus_talk",            sipc_mbus_talk},
  {"mbus_codec",           sipc_mbus_codec},
  {"mbus_packetization",   sipc_mbus_packetization},
  {(char *) NULL,          (Tcl_CmdProc *) NULL}
};

int Sipcmbus_Init(Tcl_Interp *interp)
{
    MbusCmd *cmdPtr;

    if (Tcl_PkgRequire(interp, "Tcl", TCL_VERSION, 1) == NULL) {
        return TCL_ERROR;
    }

    for (cmdPtr = commands; cmdPtr->name != NULL; cmdPtr++) {
        Tcl_CreateCommand(interp, cmdPtr->name, cmdPtr->cmdProc,
                (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
    }

    return TCL_OK;
}

/*
 * This function is to initialize mbus library
 * arguments: no
 */
int 
sipc_mbus_control(ClientData dummy, Tcl_Interp *interp, int argc, char **argv)
{
	assert(argc);
    if (argc > 1) {
        Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0], NULL);
        return TCL_ERROR;
    }

  /* The part to start the MBUS thread */
#ifdef DEBUG
  fprintf(stderr, "sipc_mbus_control\n");
#endif
  sip_mbus_control();
  return TCL_OK;
}

/*
 * When sipc intend to change remote ip address, 
 * src/dst port and ttl
 * arguments: ip-address source-port destination-port ttl
 */
int
sipc_mbus_rtp_addr(ClientData dummy, Tcl_Interp *interp, int argc, char **argv)
{
  if (argc != 5) {
    Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
       " <remote IP> <src port> <dest port> <ttl>", NULL);
    return TCL_ERROR;
  }

  /* Call the function to send rtp.addr, argv[1] will be remote IP ... */
#ifdef DEBUG
  fprintf(stderr, "sipc_mbus_rtp_addr %s:%s:%s:%s\n", 
		  argv[1], argv[2], argv[3], argv[4]);
#endif

  /* argv[1]: the address to which you want to change
   * argv[2]: source port
   * argv[3]: destination port
   * argv[4]: ttl
   */
  rtp_addr_change(&argv[1]);
  send_rtp_query();

  return TCL_OK;
}

/* 
 * to toggle mute/talk of RAT
 * arguments: 0/1
 *   0: talk
 *   1: mute
 */
int
sipc_mbus_talk(ClientData dummy, Tcl_Interp *interp, int argc, char **argv)
{
    if (argc != 2) {
        Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
            " <0|1>", NULL);
        return TCL_ERROR;
    }

  /* Call the function to set 'talk' or 'mute', send rtp.soruce.mute */
#ifdef DEBUG
  fprintf(stderr, "sipc_mbus_talk %s\n", argv[1]);
#endif

  /* argv[1]: 1: mute, 0: talk */
  rat_talk(argv[1]);
  return TCL_OK;
}

/* 
 * change the codec of RAT in use
 * arguments: codec
 *  codec: l16, pcmu, pcma, dvi, gsm, lpc
 */
int
sipc_mbus_codec(ClientData dummy, Tcl_Interp *interp, int argc, char **argv)
{
    if (argc != 2) {
        Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
            " <codec name>", NULL);
        return TCL_ERROR;
    }

  /* Call the function to send tool.rat.codec */
#ifdef DEBUG
  fprintf(stderr, "sipc_mbus_codec %s\n", argv[1]);
#endif

  // argv[1]: codec name among l16, pcmu, dvi, gsm, lpc
  tool_rat_codec(argv[1]);
  return TCL_OK;
}

/* 
 * choose the rate
 */
int
sipc_mbus_packetization (ClientData dummy, Tcl_Interp *interp, int argc, char **argv)
{
    if (argc != 2) {
        Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
            " [1|2|4|8]", NULL);
        return TCL_ERROR;
    }

  /* Call the function to send tool.rat.rate */
#ifdef DEBUG
  fprintf(stderr, "sipc_mbus_packetization %s\n", argv[1]);
#endif
  /* rate: 1, 2, 4, 8 */
  tool_codec_rate(argv[1]);

  return TCL_OK;
}
